﻿using System;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Windows.Media.Imaging;
using Ozeki.Media;
using Ozeki.Camera;
using System.Windows.Forms;
/// <summary>
//Based on blog post from Chris van Beek
/// </summary>
namespace BenCapCam
{
    class IpCamController
    {
        public DrawingImageProvider ImageProvider { get; private set; }
        public SnapshotHandler Snapshot { get; private set; }
        private MPEG4Recorder _mpeg4Recorder;
        private int cntpop = 0;
        private string _url;
        private string _videoRelative = "/videostream.cgi?resolution=32&rate=0";
        private bool _panning;
        private string _panningRelativeUri = "/decoder_control.cgi?command={0}";
        private HttpClient _client;
        private AutomaticMultiPartReader _reader;
        private BitmapImage _currentFrame;
        private frmLogForm myLog;

        /**********************************
         *Function:  CurrentFrame
         *Parameters: null
         *Results: Returns _currentFrame 
         **********************************/
        public BitmapImage CurrentFrame
        {
            get { return _currentFrame; }
        }
        /********************************************************************
         *Function: IpCamController
         *Parameters: string url, string username, string passwrod
         *Results: Contstructs IpCamController which connects to the camera
         *******************************************************************/
        public IpCamController(string url, string username, string password)
        {
            _url = url;
            WebRequestHandler handler = new WebRequestHandler();
            handler.Credentials = new NetworkCredential(username, password);
            _client = new HttpClient(handler);
            _client.BaseAddress = new Uri(_url);
            _client.Timeout = TimeSpan.FromMilliseconds(-1);

            myLog = new frmLogForm();
            myLog.Show();
            myLog.AddText(_url.ToString() + '\n');
            myLog.AddText(handler.Credentials.ToString() + '\n');
            myLog.AddText(_client.ToString() + '\n');
            myLog.AddText(_client.BaseAddress.ToString());
            myLog.AddText(_client.Timeout.ToString());
        }
        /************************************************************
         *Function: SendPanCommand
         *Parameters: Int commandNumber
         *Results: Based on number sent, sends pan command to camera 
         ************************************************************/
        private async void SendPanCommand(int commandNumber)
        {
            HttpResponseMessage result;
            result = await _client.GetAsync(string.Format(_panningRelativeUri, commandNumber));
            result.EnsureSuccessStatusCode();
        }
        /***************************************************
         *Function: _reader_PartReady
         *Parameters: object sender, PartReadyEventArgs e
         *Results: Get evetns back to UI thread 
         ****************************************************/
        private void _reader_PartReady(object sender, PartReadyEventArgs e)
        {
            Stream frameStream = new MemoryStream(e.Part);
            App.Current.Dispatcher.Invoke(new Action(() =>
            {
                _currentFrame = new BitmapImage();
                _currentFrame.BeginInit();
                _currentFrame.StreamSource = frameStream;
                _currentFrame.EndInit();
                OnImageReady();
            }));
        }
        /*******************************************************************************************
         *Function: StartProcessing
         *Parameters: null
         *Results: Starts Processing the IpCamController, connecting to camera and sending commands 
         ********************************************************************************************/
        public async void StartProcessing()
        {
            try
            {
                HttpResponseMessage resultMessage = await _client.GetAsync(_videoRelative, HttpCompletionOption.ResponseHeadersRead);
                //because of the configure await the rest of this method happens on a background thread.
                resultMessage.EnsureSuccessStatusCode();
                // check the response type
                if (!resultMessage.Content.Headers.ContentType.MediaType.Contains("multipart"))
                {
                    throw new ArgumentException("The camera did not return a mjpeg stream");
                }
                else
                {
                    _reader = new AutomaticMultiPartReader(new MultiPartStream(await resultMessage.Content.ReadAsStreamAsync()));
                    _reader.PartReady += _reader_PartReady;
                    _reader.StartProcessing();
                }
            }
            catch
            {
                MessageBox.Show("Could not connect to video stream");
            }
        }
        /************************************************************************
         *Function: PicSave
         *Parameters: null
         *Results: Used by a timer to save currentFrames to the correct folder 
         ************************************************************************/
        public void PicSave()
        {
            try
            {
                DateTime DTN = DateTime.Now;
                string fileName = "C:/Users/bpt-g/Desktop/capcam/" + DTN.Year + DTN.Month + DTN.Day + DTN.Hour + DTN.Minute + DTN.Second + ".bmp";
                using (MemoryStream outStream = new MemoryStream())
                {
                    BitmapEncoder enc = new BmpBitmapEncoder();
                    enc.Frames.Add(BitmapFrame.Create(_currentFrame));
                    enc.Save(outStream);
                    System.Drawing.Bitmap bitmap = new System.Drawing.Bitmap(outStream);
                    bitmap.Save(fileName);
                }
            }
            catch//just in case not connected to the camera
            {
                if (cntpop == 0)
                    MessageBox.Show("Could not save feed");
                cntpop++;
            }
        }
        //Below are the various Pan Commands
        //**************************************************************
        public void PanRight()
        {
            int command = _panning ? 5 : 4;
            SendPanCommand(command);
            _panning = !_panning;
        }

        public void PanLeft()
        {
            int command = _panning ? 7 : 6;
            SendPanCommand(command);
            _panning = !_panning;
        }

        public void PanDown()
        {
            int command = _panning ? 3 : 2;
            SendPanCommand(command);
            _panning = !_panning;
        }

        public void PanUp()
        {
            int command = _panning ? 1 : 0;
            SendPanCommand(command);
            _panning = !_panning;
        }
        //*******************************************************************
        public void StopProcessing()
        {
            _reader.StopProcessing();
        }
        public void StopProcessing2()
        {
            _reader.StopProcessing2();
        }
        public void StartVideoFeedAgain()
        {
            _reader.Start();
        }
        protected void OnImageReady()
        {
            if (ImageReady != null)
            {
                ImageReady(this, new ImageReadyEventArsgs() { Image = _currentFrame/*save*/ });
            }
        }

        public event EventHandler<ImageReadyEventArsgs> ImageReady;

    }
}
